@@ -51,7 +51,7 @@ bclogit.default <- function(formula = NULL,
 
   # Use copy of X if needed or just use X directly
   if (test_data_frame(X, types = c("numeric", "integer", "factor", "logical"))) {
-    data_mat <- model.matrix(~ ., data = X)
+    data_mat <- model.matrix(~., data = X)
     if ("(Intercept)" %in% colnames(data_mat)) {
       data_mat <- data_mat[, colnames(data_mat) != "(Intercept)", drop = FALSE]
     }
@@ -96,14 +96,22 @@ bclogit.default <- function(formula = NULL,
     stop("Treatment must be binary 0 or 1.")
   }
   if (is.null(treatment_name)) {
-    treatment_name <- paste(deparse(substitute(treatment)), collapse = "")
+    orig_trt <- substitute(treatment)
+    if (is.name(orig_trt)) {
+      treatment_name <- as.character(orig_trt)
+    } else {
+      treatment_name <- paste(deparse(orig_trt), collapse = "")
+      if (nchar(treatment_name) > 30 || grepl("^c\\(", treatment_name) || grepl("^[0-9\\.\\-]+", treatment_name)) {
+        treatment_name <- "treatment"
+      }
+    }
   }
 
   # Filter to keep only strata with exactly 2 observations
   strata_counts <- table(strata)
   valid_strata <- names(strata_counts)[strata_counts == 2]
   keep_idx <- strata %in% as.numeric(valid_strata)
-  
+
   if (!all(keep_idx)) {
     y <- y[keep_idx]
     data_mat <- data_mat[keep_idx, , drop = FALSE]
@@ -189,7 +197,6 @@ bclogit.default <- function(formula = NULL,
     }
     if (concordant_method == "GEE") {
       gee_df <- data.frame(y_concordant, treatment_concordant, strata_concordant)
-      gee_df$X_concordant <- X_concordant
       concordant_model <- geepack::geeglm(
         y_concordant ~ treatment_concordant + X_concordant,
         id = strata_concordant,
@@ -225,7 +232,11 @@ bclogit.default <- function(formula = NULL,
       }
 
       # Identify target names: treatment + X columns
-      target_names <- c("treatment_concordant", paste0("X_concordant", X_model_matrix_col_names))
+      if (ncol(X_concordant) == 1) {
+        target_names <- c("treatment_concordant", "X_concordant")
+      } else {
+        target_names <- c("treatment_concordant", paste0("X_concordant", X_model_matrix_col_names))
+      }
 
       # Construct target vector
       K_stan <- ncol(X_diffs_discordant) + 1
@@ -330,6 +341,39 @@ bclogit.default <- function(formula = NULL,
           Sigma_con[, 1] <- 0
           Sigma_con[1, 1] <- prior_variance_treatment
           Sigma_con <- (Sigma_con + t(Sigma_con)) / 2
+        } else if (concordant_method == "GLMM") {
+          warning("GLMM prior covariance is not positive definite (common with boundary variance estimates or convergence failures). Falling back to GLM covariance for the prior.")
+          glm_fallback <- glm(y_concordant ~ treatment_concordant + X_concordant, family = "binomial")
+          full_b_fb <- coef(glm_fallback)
+          full_S_fb <- vcov(glm_fallback)
+          b_con <- numeric(K_stan)
+          Sigma_con <- diag(100, K_stan)
+          for (i in seq_along(target_names)) {
+            t_name <- target_names[i]
+            if (t_name %in% names(full_b_fb)) {
+              val <- full_b_fb[t_name]
+              if (!is.na(val)) b_con[i] <- val
+            }
+          }
+          vcov_names_fb <- rownames(full_S_fb)
+          valid_names_fb <- names(full_b_fb)[names(full_b_fb) %in% target_names]
+          for (r_name in valid_names_fb) {
+            if (r_name %in% vcov_names_fb) {
+              target_idx_r <- match(r_name, target_names)
+              for (c_name in valid_names_fb) {
+                if (c_name %in% vcov_names_fb) {
+                  target_idx_c <- match(c_name, target_names)
+                  val <- full_S_fb[r_name, c_name]
+                  if (!is.na(val)) Sigma_con[target_idx_r, target_idx_c] <- val
+                }
+              }
+            }
+          }
+          b_con[1] <- 0
+          Sigma_con[1, ] <- 0
+          Sigma_con[, 1] <- 0
+          Sigma_con[1, 1] <- prior_variance_treatment
+          Sigma_con <- (Sigma_con + t(Sigma_con)) / 2
         } else {
           warning("Prior covariance is not positive definite. Replacing with independent and diffuse prior.")
           Sigma_con <- diag(100, nrow(Sigma_con))
@@ -468,4 +512,3 @@ bclogit.default <- function(formula = NULL,
   class(res) <- c("bclogit", "list")
   return(res)
 }
-
